<?php

declare(strict_types=1);

namespace Ifacesoft\Ice\Query\Application\UseCase\Once;

use Ifacesoft\Ice\Core\Domain\ValueObject\Scalar\StringValue\Generic;
use Ifacesoft\Ice\Core\Domain\ValueObject\Scalar\StringValue\Type;
use Ifacesoft\Ice\Data\Domain\ValueObject\Scalar\Primitive\StringValue;
use Ifacesoft\Ice\Data\Interfaces\ValueObject\Scalar\StringValue\XmlStringInterface;
use Ifacesoft\Ice\Generator\Domain\Abstracts\AbstractClass;
use Ifacesoft\Ice\Generator\Domain\Collection\Dto\Class\PropertyCollection;
use Ifacesoft\Ice\Generator\Domain\Dto\Class\Property;
use Ifacesoft\Ice\Generator\Domain\Interfaces\GeneratorInterface;
use Ifacesoft\Ice\Generator\Domain\Model\Class\Php7\Collection;
use Ifacesoft\Ice\Generator\Domain\Model\Class\Php7\Struct;
use Ifacesoft\Ice\Render\Domain\Interfaces\TemplaterInterface;
use JsonException;
use ReflectionException;
use SimpleXMLElement;

class GenerateDtosDbwrenchUseCase
{
    private GeneratorInterface $generator;
    private TemplaterInterface $templater;

    public function __construct(
        GeneratorInterface $generator,
        TemplaterInterface $templater
    ) {
        $this->generator = $generator;
        $this->templater = $templater;
    }

    /**
     * @throws JsonException
     * @throws ReflectionException
     */
    public function __invoke(XmlStringInterface $xmlString): void
    {
        /** @var array<string, mixed> $properties */
        $properties = $this->xmlToArray($xmlString->toSimpleXmlElement()->Sch); // @phpstan-ignore property.nonObject

        $class = Type::create('Ifacesoft\Ice\Query\\', 'Domain\Dto\\', 'Dbwrench');

        $this
            ->create($class, $properties)
            ->generate($this->generator, $this->templater)
        ;
    }

    /**
     * @param array<string, mixed> $properties
     *
     * @throws ReflectionException
     */
    public function create(Type $class, array $properties): AbstractClass
    {
        $propertyCollection = new PropertyCollection();

        foreach ($properties as $propertyName => $value) {
            if (is_array($value)) {
                if (empty($value)) {
                    continue;
                }

                $propertyClassName = ucfirst((new StringValue($propertyName))->toCamelCase()->getValue());

                if (is_int(key($value))) {
                    $collectionClass = $class->getValue() . '\\' . $propertyClassName . 'Collection';

                    $structType = new Type($collectionClass . '\\' . $propertyClassName);

                    $collectionType = new Type(
                        $collectionClass,
                        new Generic(null, new Type('int'), $structType)
                    );

                    (new Collection($collectionType))
                        ->generate($this->generator, $this->templater)
                    ;

                    $this
                        ->create($structType, $value[0])  // todo: пробежать по всем и собрать максиму полей (в первом объекте могут быть не все поля)
                        ->generate($this->generator, $this->templater)
                    ;

                    $propertyCollection = $propertyCollection->add(
                        new Property(
                            $propertyName,
                            $collectionType
                        )
                    );
                } else {
                    $structType = new Type($class->getValue() . '\\' . $propertyClassName);

                    $this
                        ->create($structType, $value)
                        ->generate($this->generator, $this->templater)
                    ;

                    $propertyCollection = $propertyCollection->add(
                        new Property(
                            $propertyName,
                            $structType
                        )
                    );
                }
            } else {
                $propertyCollection = $propertyCollection->add(
                    new Property(
                        $propertyName,
                        new Type(is_numeric($value) ? 'int' : 'string')
                    )
                );
            }
        }

        return new Struct($class, $propertyCollection);
    }

    /**
     * @return array<mixed>|string
     */
    private function xmlToArray(SimpleXMLElement $xmlObject)
    {
        $array = [];

        if ($xmlObject->attributes()) {
            foreach ($xmlObject->attributes() as $key => $value) {
                $array['attributes'][$key] = (string) $value;
            }
        }

        if ($xmlObject->children()) {
            foreach ($xmlObject->children() as $key => $child) {
                $array[$key][] = $this->xmlToArray($child);
            }

            foreach ($array as &$value) {
                if (is_array($value) && 1 === count($value) && !is_array(reset($value))) {
                    $value = reset($value);
                }
            }
            unset($value);
        } else {
            $array = (string) $xmlObject;
        }

        return $array;
    }
}
