<?php

declare(strict_types=1);

namespace Ifacesoft\Ice\Query\Domain\Dto\Dbwrench\TblCollection;

use Ifacesoft\Ice\Data\Domain\Abstracts\AbstractStruct;
use Ifacesoft\Ice\Query\Domain\Dto\Dbwrench\TblCollection\Tbl\Attributes;
use Ifacesoft\Ice\Query\Domain\Dto\Dbwrench\TblCollection\Tbl\ClCollection;
use Ifacesoft\Ice\Query\Domain\Dto\Dbwrench\TblCollection\Tbl\ClCollection\Cl;
use Ifacesoft\Ice\Query\Domain\Dto\Dbwrench\TblCollection\Tbl\FkCollection;
use Ifacesoft\Ice\Query\Domain\Dto\Dbwrench\TblCollection\Tbl\FkCollection\Fk;
use Ifacesoft\Ice\Query\Domain\Dto\Dbwrench\TblCollection\Tbl\IdxCollection;
use Ifacesoft\Ice\Query\Domain\Dto\Dbwrench\TblCollection\Tbl\IdxCollection\Idx;
use Ifacesoft\Ice\Query\Domain\Dto\Dbwrench\TblCollection\Tbl\PkCollection;
use Ifacesoft\Ice\Query\Domain\Dto\Dbwrench\TblCollection\Tbl\PkCollection\Pk;
use ReflectionException;

final class Tbl extends AbstractStruct
{
    private Attributes $attributes;

    private string $TblOpts;

    /**
     * @var PkCollection<int, Pk>
     */
    private PkCollection $Pk;

    /**
     * @var ClCollection<int, Cl>
     */
    private ClCollection $Cl;

    /**
     * @var FkCollection<int, Fk>
     */
    private ?FkCollection $Fk;

    /**
     * @var ?IdxCollection<int, Idx>
     */
    private ?IdxCollection $Idx;

    private ?string $UniqueConstraints;

    private ?string $SchTrHis;

    /**
     * @param PkCollection<int, Pk>    $Pk
     * @param ClCollection<int, Cl>    $Cl
     * @param ?FkCollection<int, Fk>   $Fk
     * @param ?IdxCollection<int, Idx> $Idx
     *
     * @throws ReflectionException
     */
    public function __construct(Attributes $attributes, string $TblOpts, PkCollection $Pk, ClCollection $Cl, ?FkCollection $Fk = null, ?IdxCollection $Idx = null, ?string $UniqueConstraints = null, ?string $SchTrHis = null)
    {
        $this->attributes = $attributes;
        $this->TblOpts = $TblOpts;
        $this->Pk = $Pk;
        $this->Cl = $Cl;
        $this->Fk = $Fk;
        $this->Idx = $Idx;
        $this->UniqueConstraints = $UniqueConstraints;
        $this->SchTrHis = $SchTrHis;

        parent::__construct();
    }

    public function getAttributes(): Attributes
    {
        return $this->attributes;
    }

    public function getTblOpts(): string
    {
        return $this->TblOpts;
    }

    /**
     * @return PkCollection<int, Pk>
     */
    public function getPk(): PkCollection
    {
        return $this->Pk;
    }

    /**
     * @return ClCollection<int, Cl>
     */
    public function getCl(): ClCollection
    {
        return $this->Cl;
    }

    /**
     * @return ?FkCollection<int, Fk>
     */
    public function getFk(): ?FkCollection
    {
        return $this->Fk;
    }

    /**
     * @return ?IdxCollection<int, Idx>
     */
    public function getIdx(): ?IdxCollection
    {
        return $this->Idx;
    }

    public function getUniqueConstraints(): ?string
    {
        return $this->UniqueConstraints;
    }

    public function getSchTrHis(): ?string
    {
        return $this->SchTrHis;
    }
}
