<?php

declare(strict_types=1);

namespace Ifacesoft\Ice\Query\Domain\Dto\Table;

use Ifacesoft\Ice\Core\Domain\ValueObject\Scalar\StringValue\Type;
use Ifacesoft\Ice\Data\Domain\Abstracts\AbstractStruct;
use Ifacesoft\Ice\Data\Domain\ValueObject\Scalar\Primitive\StringValue;
use Ifacesoft\Ice\Generator\Domain\Interfaces\Dto\Class\PropertyInterface;
use Ifacesoft\Ice\Query\Domain\Dto\Table\Column\Join;
use Ifacesoft\Ice\Query\Domain\Model\Table\Column\DataType;
use ReflectionException;

final class Column extends AbstractStruct implements PropertyInterface
{
    private string $name;
    private string $postfix;
    private DataType $dataType;
    private ?string $default;
    private bool $pk;
    private ?Join $fk;

    public function __construct(
        string  $name,
        string  $dataType,
        bool    $nullable,
        int     $length,
        ?string $default,
        bool    $pk,
        ?Join   $fk
    ) {
        $this->name = $name;
        $this->postfix = $this->detectPostfix($name);
        $this->dataType = new DataType($dataType, $nullable, $length);
        $this->default = $default;
        $this->pk = $pk;
        $this->fk = $fk;

        parent::__construct();
    }

    public function getAlias(): string
    {
        return (new StringValue($this->getName()))->trim(null, $this->getPostfix())->toCamelCase()->getValue();
    }

    public function getName(): string
    {
        return $this->name;
    }

    public function getPostfix(): string
    {
        return $this->postfix;
    }

    public function getDataType(): DataType
    {
        return $this->dataType;
    }

    public function getDefault(): ?string
    {
        return $this->default;
    }

    public function isPk(): bool
    {
        return $this->pk;
    }

    public function getFk(): ?Join
    {
        return $this->fk;
    }

    /**
     * @throws ReflectionException
     */
    public function getType(): Type
    {
        return $this->dataType->getPhpType();
    }

    private function detectPostfix(string $columnName): string
    {
        static $postfixes = ['__fk'];

        foreach ($postfixes as $postfix) {
            if ((new StringValue($columnName))->isEndsWith($postfix)) {
                return $postfix;
            }
        }

        return '';
    }
}
