<?php

declare(strict_types=1);

namespace Ifacesoft\Ice\Query\Domain\Model\Table\Column;

use Ifacesoft\Ice\Core\Domain\ValueObject\Scalar\StringValue\Type;
use Ifacesoft\Ice\Data\Domain\Abstracts\AbstractStruct;
use ReflectionException;
use RuntimeException;

final class DataType extends AbstractStruct
{
    private string $name;
    private bool $nullable;
    private int $length;

    public function __construct(string $name, bool $nullable, int $length)
    {
        $this->name = $name;
        $this->nullable = $nullable;
        $this->length = $length;

        parent::__construct();
    }

    public function isNullable(): bool
    {
        return $this->nullable;
    }

    public function getLength(): int
    {
        return $this->length;
    }

    /**
     * @see https://doctrine-dbal-rtd.readthedocs.io/en/latest/reference/types.html
     */
    public function getDoctrineType(): string
    {
        return match (strtoupper($this->getName())) {
            'TINYINT' => 'boolean',
            'INTEGER' => 'integer',
            'VARCHAR' => 'string',
            'DATE', 'DATETIME' => 'date',
            default => throw new RuntimeException('DataType ' . $this->getName() . ' not mapped to doctrine type'),
        };
    }

    public function getName(): string
    {
        return $this->name;
    }

    /**
     * @see https://doctrine-dbal-rtd.readthedocs.io/en/latest/reference/types.html
     *
     * @throws ReflectionException
     */
    public function getPhpType(): Type
    {
        return new Type(
            match (strtoupper($this->getName())) {
                'TINYINT' => 'bool',
                'INTEGER' => 'int',
                'VARCHAR' => 'string',
                'DATE', 'DATETIME' => 'DataTimeImmutable',
                default => throw new RuntimeException('DataType ' . $this->getName() . ' not mapped to php type'),
            }
        );
    }
}
