<?php

declare(strict_types=1);

namespace Ifacesoft\Ice\Query\Domain\ValueObject;

use Ifacesoft\Ice\Core\Domain\Helper\Model\ModuleHelper;
use Ifacesoft\Ice\Core\Domain\Model\Module;
use Ifacesoft\Ice\Core\Domain\ValueObject\Scalar\StringValue\Type;
use Ifacesoft\Ice\Data\Domain\ValueObject\Scalar\Primitive\StringValue;
use ReflectionException;

final class Table extends StringValue
{
    private ?Module $iceModule;
    private ?string $prefix;

    /**
     * @throws ReflectionException
     */
    public function __construct(string $value)
    {
        /** @var Module $module */
        foreach (ModuleHelper::getAll() as $module) {
            if ($prefixes = $module->getPrefixes()) {
                /** @var string $prefix */
                foreach ($prefixes as $prefix) {
                    if ($prefix && 0 === strpos($value, $prefix)) {
                        $this->prefix = $prefix;
                        $this->iceModule = $module;

                        parent::__construct($value);

                        return;
                    }
                }
            }
        }

        $this->prefix = null;
        $this->iceModule = null;
    }

    public function getType(string $namespace, string $classNamePostfix = ''): Type
    {
        $moduleNamespace = '';

        if ($module = $this->getIceModule()) {
            $moduleNamespace = $module->getNamespace();
        }

        return Type::create($moduleNamespace, $namespace, $this->getClassName() . $classNamePostfix);
    }

    public function getIceModule(): ?Module
    {
        return $this->iceModule;
    }

    public function getClassName(): string
    {
        $rawClassName = $this->getValue();

        if ($prefix = $this->getPrefix()) {
            $rawClassName = substr($rawClassName, strlen($prefix));
        }

        return ucfirst((new StringValue($rawClassName))->toCamelCase()->getValue());
    }

    public function getPrefix(): ?string
    {
        return $this->prefix;
    }
}
